#include <cstring>
#include <iostream>

#include "MathTools.h"
#include "SinusTable.h"
#include "StopWatch.h"


//============================================================================
// Implementation of Tools.h
//============================================================================

//----------------------------------------------------------------------------
int myRand(int limit)
{
    return (int)(1.0 * limit * rand() / (RAND_MAX + 1.0));
}



//============================================================================
// Implementation of MathTools.h
//============================================================================

//----------------------------------------------------------------------------
double MATH_TOOLS::getAngle(double x, double y)
{
    double angle = (atan2(y, -x) * 360.0 / (2.0*M_PI)) - 90.0;
    if (angle < 0.0)  angle += 360.0;
    return angle;
}

//----------------------------------------------------------------------------
int MATH_TOOLS::convertAngle(int angle)
{
    if (angle < 180)
    {
        return 180 - angle;
    }
    else
    {
        return 360 + 180 - angle;
    }
}



//============================================================================
// Implementation of SinusTable.h
//============================================================================

//----------------------------------------------------------------------------
#define SINUS_TABLE_SCALE 1024

//----------------------------------------------------------------------------
SinusTable SinusTable::sm_instance;
int SinusTable::sm_scale = SINUS_TABLE_SCALE;
int SinusTable::sm_sinTable[360];
int SinusTable::sm_cosTable[360];

//----------------------------------------------------------------------------
SinusTable::SinusTable()
{
    for (int i=0; i<360; i++)
    {
        int s = (int)(SINUS_TABLE_SCALE * ::sin(i*2.0*M_PI/360.0));
        sm_sinTable[i] = s;
        sm_cosTable[(i+270)%360] = s;
    }
}

//----------------------------------------------------------------------------
SinusTable::~SinusTable()
{
}



//============================================================================
// Implementation of StopWatch.h
//============================================================================

//----------------------------------------------------------------------------
StopWatch::StopWatch(const char *name)
{
    m_name = name;

    memset(&m_start, 0, sizeof(m_start));
    memset(&m_stop, 0, sizeof(m_stop));
    memset(&m_total, 0, sizeof(m_total));
}

//----------------------------------------------------------------------------
StopWatch::~StopWatch()
{
    m_name = NULL;
}

//----------------------------------------------------------------------------
void StopWatch::start()
{
    (void)gettimeofday(&m_start, NULL);
}

//----------------------------------------------------------------------------
void StopWatch::stop()
{
    (void)gettimeofday(&m_stop, NULL);

    m_total.tv_sec += m_stop.tv_sec - m_start.tv_sec;
    m_total.tv_usec += m_stop.tv_usec - m_start.tv_usec;

    if (m_total.tv_usec < 0)
    {
        m_total.tv_sec--;
        m_total.tv_usec += 1000000;
    }
    else if (m_total.tv_usec > 1000000)
    {
        m_total.tv_sec++;
        m_total.tv_usec -= 1000000;
    }
}

//----------------------------------------------------------------------------
int StopWatch::compare(const StopWatch &other) const
{
    if (m_total.tv_sec < other.m_total.tv_sec)
    {
        return -1;
    }
    if (m_total.tv_sec > other.m_total.tv_sec)
    {
        return 1;
    }
    if (m_total.tv_usec < other.m_total.tv_usec)
    {
        return -1;
    }
    if (m_total.tv_usec > other.m_total.tv_usec)
    {
        return 1;
    }

    return 0;
}

//----------------------------------------------------------------------------
std::ostream &operator<<(std::ostream &os, const StopWatch &s)
{
    return os << s.m_name << ": "
              << s.m_total.tv_sec << " sec, "
              << s.m_total.tv_usec << " usec";
}
